#region Using Statements

using System;
using System.IO;
using System.Threading;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using pipa.Menus;
using pipa.Scene;

#endregion

namespace pipa
{
    internal class GameplayScreen : GameScreen
    {
        private ContentManager content;
        private SpriteFont gameFont;
        private Level activeLevel;

        private new bool IsActive
        {
            get { return base.IsActive; }
        }

        public GameplayScreen()
        {
            TransitionOnTime = TimeSpan.FromSeconds(1.5);
            TransitionOffTime = TimeSpan.FromSeconds(0.5);
        }

        public override void LoadContent()
        {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");

            gameFont = content.Load<SpriteFont>("menu/menufont");

            // initiate level
            LoadNextLevel();
            activeLevel.LoadContent();

            ScreenManager.Game.ResetElapsedTime();
        }

        public override void UnloadContent()
        {
            content.Unload();
        }

        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                    bool coveredByOtherScreen)
        {
            if (IsActive)
            {
                activeLevel.Update(gameTime);
            }

            base.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);
        }


        /// <summary>
        /// Lets the game respond to player input. Unlike the Update method,
        /// this will only be called when the gameplay screen is active.
        /// </summary>
        public override void HandleInput(GameTime gameTime, InputState input)
        {
            if (input == null)
                throw new ArgumentNullException("input");

            if (input.IsPauseGame())
            {
                ScreenManager.AddScreen(new PauseMenuScreen());
                return;
            }

            if(activeLevel.isGameFinished && (input.IsMenuSelect() || input.IsMenuCancel()))
            {
                if(activeLevel.hasReachedExit)
                {
                    activeLevel.Dispose();
                    ScreenManager.AddScreen(new EndScreen());
                }
                else
                {
                    activeLevel.Dispose();
                    
                    LoadNextLevel();
                    activeLevel.LoadContent();
                    ScreenManager.Game.ResetElapsedTime();
                    return;
                }
            }

            activeLevel.HandleInput(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            ScreenManager.GraphicsDevice.Clear(ClearOptions.Target,
                                               Color.CornflowerBlue, 0, 0);

            SpriteBatch spriteBatch = ScreenManager.SpriteBatch;

            // draw level
            activeLevel.Draw(gameTime, spriteBatch);

            // If the game is transitioning on or off, fade it out to black.
            if (TransitionPosition > 0)
                ScreenManager.FadeBackBufferToBlack(255 - TransitionAlpha);
        }

        private void LoadNextLevel()
        {
            // Find the path of the next level.
            string levelPath;
            int levelIndex = 1;

            // Loop here so we can try again when we can't find a level.
            while (true)
            {
                // Try to find the next level. They are sequentially numbered txt files.
                levelPath = String.Format("levels/level_{0}.txt", levelIndex++);
                levelPath = Path.Combine(StorageContainer.TitleLocation, "Content/" + levelPath);
                if (File.Exists(levelPath))
                    break;

                // If there isn't even a level 0, something has gone wrong.
                if (levelIndex == 1)
                    throw new Exception("No levels found.");

                // Whenever we can't find a level, start over again at 0.
                levelIndex = -1;
            }

            // Unloads the content for the current level before loading the next one.
            if (activeLevel != null)
                activeLevel.Dispose();

            // Load the level.
            activeLevel = new Level(this.ScreenManager, levelPath);
        }
    }
}