﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace pipa.Scene
{
    public class Level
    {
        private ScreenManager screenManager;
        private Camera2D camera;

        public Player[] Player;

        public const int foregroundTileSize = 512;
        private Texture2D foregroundTexture;
        private SpriteSheet foregroundSheet;
        private TileGrid foregroundLayer;

        public const int backgroundTileSize = 512;
        private Texture2D backgroundTexture;
        private SpriteSheet backgroundSheet;
        private TileGrid backgroundLayer;

        private const int elementTextureCount = 1;
        private int elementColumns;
        private int elementRows;
        private Texture2D elementTexture;
        private SpriteSheet elementSheet;
        private TileGrid elementLayer;

        private Rope rope;
        private TargetPoint targetPoint;

        private string levelPath;
        private TimeSpan levelTime = TimeSpan.FromSeconds(182);
        private readonly TimeSpan warningTime = TimeSpan.FromSeconds(30);
        private readonly TimeSpan helpTime = TimeSpan.FromSeconds(180);
        private SpriteFont hudFont;

        private Texture2D winOverlay;
        private Texture2D loseOverlay;
        private Texture2D escapeOverlay;

        private const float MovementRate = 500f;
        public bool hasReachedExit;
        public bool isGameFinished;
        public bool hasGameStarted;

        public ContentManager Content
        {
            get { return screenManager.Content; }
        }

        public Level(ScreenManager screenManager, string path)
        {
            this.screenManager = screenManager;
            this.levelPath = path;
        }

        public void LoadContent()
        {
            Player = new Player[2];
            Player[0] = new Player(this, Vector2.Zero, 0);
            Player[1] = new Player(this, Vector2.Zero, 1);

            Player[0].LoadContent();          
            Player[0].Reset(Vector2.Zero);        
            Player[1].LoadContent();       
            Player[1].Reset(Vector2.Zero);

            rope = new Rope(screenManager);

            targetPoint = new TargetPoint(screenManager);
            targetPoint.LoadContent();
            targetPoint.Position = new Vector2(384, -200);

            LoadTiles();
            LoadBackgroundLayer();
            LoadForegroundLayer();

            hudFont = screenManager.Content.Load<SpriteFont>("scene/hudFont");
            escapeOverlay = screenManager.Content.Load<Texture2D>("overlays/ui_overlay_escape");
            winOverlay = screenManager.Content.Load<Texture2D>("overlays/ui_overlay_win");
            loseOverlay = screenManager.Content.Load<Texture2D>("overlays/ui_overlay_busted");

            camera = new Camera2D();
            camera.Position = new Vector2(foregroundTileSize, 4*foregroundTileSize - 384);
            camera.Rotation = 0f;
            camera.Zoom = 1f;
            camera.MoveUsingScreenAxis = true;
            CameraChanged();
        }

        private void LoadBackgroundLayer()
        {
            backgroundTexture = screenManager.Content.Load<Texture2D>("scene/background");
            
            backgroundSheet = new SpriteSheet(backgroundTexture);

            backgroundSheet.AddSourceSprite(1, new Rectangle(0, 0, backgroundTileSize, backgroundTileSize));
            backgroundSheet.AddSourceSprite(2,
                                            new Rectangle(backgroundTileSize, 0, backgroundTileSize, backgroundTileSize));
            backgroundSheet.AddSourceSprite(3,
                                            new Rectangle(0, backgroundTileSize, backgroundTileSize, backgroundTileSize));
            backgroundSheet.AddSourceSprite(4,
                                            new Rectangle(backgroundTileSize, backgroundTileSize, backgroundTileSize,
                                                          backgroundTileSize));
            backgroundSheet.AddSourceSprite(5,
                                            new Rectangle(0, 2 * backgroundTileSize, backgroundTileSize,
                                                          backgroundTileSize));
            backgroundSheet.AddSourceSprite(6,
                                            new Rectangle(backgroundTileSize, 2 * backgroundTileSize, backgroundTileSize,
                                                          backgroundTileSize));
            backgroundSheet.AddSourceSprite(7,
                                            new Rectangle(0, 3 * backgroundTileSize, backgroundTileSize,
                                                          backgroundTileSize));
            backgroundSheet.AddSourceSprite(8,
                                            new Rectangle(backgroundTileSize, 3 * backgroundTileSize, backgroundTileSize,
                                                          backgroundTileSize));

            backgroundLayer = new TileGrid(backgroundTileSize, backgroundTileSize, 2, 10, new Vector2(0, 0),
                                           backgroundSheet, screenManager.Graphics);

            int y = 0;

            backgroundLayer.SetTile(0, y, new Tile(1));
            backgroundLayer.SetTile(1, y++, new Tile(2));
            backgroundLayer.SetTile(0, y, new Tile(3));
            backgroundLayer.SetTile(1, y++, new Tile(4));

            //for (int i = 0; i < 3; i++)
            {
                backgroundLayer.SetTile(0, y, new Tile(5));
                backgroundLayer.SetTile(1, y++, new Tile(6));
            }

            backgroundLayer.SetTile(0, y, new Tile(7));
            backgroundLayer.SetTile(1, y++, new Tile(8));
            backgroundLayer.SetTile(0, y, new Tile(7));
            backgroundLayer.SetTile(1, y++, new Tile(8));
        }

        private void LoadForegroundLayer()
        {
            foregroundTexture = screenManager.Content.Load<Texture2D>("scene/foreground");
            foregroundSheet = new SpriteSheet(foregroundTexture);

            foregroundSheet.AddSourceSprite(1, new Rectangle(0, 0, foregroundTileSize, foregroundTileSize));
            foregroundSheet.AddSourceSprite(2,
                                            new Rectangle(foregroundTileSize, 0, foregroundTileSize, foregroundTileSize));
            foregroundSheet.AddSourceSprite(3,
                                            new Rectangle(0, foregroundTileSize, foregroundTileSize, foregroundTileSize));
            foregroundSheet.AddSourceSprite(4,
                                            new Rectangle(foregroundTileSize, foregroundTileSize, foregroundTileSize,
                                                          foregroundTileSize));
            foregroundSheet.AddSourceSprite(5,
                                            new Rectangle(0, 2*foregroundTileSize, foregroundTileSize,
                                                          foregroundTileSize));
            foregroundSheet.AddSourceSprite(6,
                                            new Rectangle(foregroundTileSize, 2*foregroundTileSize, foregroundTileSize,
                                                          foregroundTileSize));
            foregroundSheet.AddSourceSprite(7,
                                            new Rectangle(0, 3*foregroundTileSize, foregroundTileSize,
                                                          foregroundTileSize));
            foregroundSheet.AddSourceSprite(8,
                                            new Rectangle(foregroundTileSize, 3*foregroundTileSize, foregroundTileSize,
                                                          foregroundTileSize));

            foregroundLayer = new TileGrid(foregroundTileSize, foregroundTileSize, 2, 10, new Vector2(0, 0),
                                           foregroundSheet, screenManager.Graphics);

            int y = 0;

            foregroundLayer.SetTile(0, y, new Tile(1));
            foregroundLayer.SetTile(1, y++, new Tile(2));
            foregroundLayer.SetTile(0, y, new Tile(3));
            foregroundLayer.SetTile(1, y++, new Tile(4));

            //for (int i = 2; i <= 8; i++)
            {
                foregroundLayer.SetTile(0, y, new Tile(5));
                foregroundLayer.SetTile(1, y++, new Tile(6));
            }

            foregroundLayer.SetTile(0, y, new Tile(7));
            foregroundLayer.SetTile(1, y++, new Tile(8));
        }

        private void LoadTiles()
        {
            elementTexture = screenManager.Content.Load<Texture2D>("scene/elements");
            elementSheet = new SpriteSheet(elementTexture);
            elementSheet.AddSourceSprite(1,
                                         new Rectangle(0, 0, elementTexture.Width/elementTextureCount,
                                                       elementTexture.Height));

            // Load the level and ensure all of the lines are the same length.
            int width;
            List<string> lines = new List<string>();
            using (StreamReader reader = new StreamReader(levelPath))
            {
                string line = reader.ReadLine();
                width = line.Length;
                while (line != null)
                {
                    lines.Add(line);
                    if (line.Length != width)
                        throw new Exception(
                            String.Format("The length of line {0} is different from all preceeding lines.", lines.Count));
                    line = reader.ReadLine();
                }
            }

            elementColumns = width;
            elementRows = lines.Count;

            // Allocate the tile grid
            float yOffset = (float) Math.Round((elementRows*elementTexture.Height)/(float) foregroundTileSize)*
                            foregroundTileSize -
                            (elementRows*elementTexture.Height);
            elementLayer = new TileGrid(64, 48, elementColumns, elementRows, new Vector2(0, yOffset), elementSheet,
                                        screenManager.Graphics);

            // Loop over every tile position,
            for (int y = 0; y < elementRows; y++)
            {
                for (int x = 0; x < elementColumns; x++)
                {
                    // to load each tile.
                    char tileType = lines[y][x];

                    switch (tileType)
                    {
                        case '#':
                            elementLayer.SetTile(x, y, new Tile(1, Tile.TileCollision.Platform));
                            break;
                        case '1':
                            Vector2 start = RectangleExtensions.GetBottomCenter(GetBounds(x, y));
                            Player[0].Position = start;
                            break;
                        case '2':
                            Vector2 start2 = RectangleExtensions.GetBottomCenter(GetBounds(x, y));
                            Player[1].Position = start2;
                            break;
                        //case 'T':
                        //    Vector2 start3 = RectangleExtensions.GetBottomCenter(GetBounds(x, y));
                        //    targetPoint.Position = start3;
                        //    break;
                    }
                }
            }
        }

        public void HandleInput(GameTime gameTime)
        {
            camera.ResetChanged();

            HandleKeyboardInput((float) gameTime.ElapsedGameTime.TotalSeconds);

            if (camera.IsChanged)
            {
                CameraChanged();
            }
        }

        public void Update(GameTime time)
        {
            #region gametime

            for (int i = 0; i < Player.Length; i++)
            {
                if (targetPoint.hasReachedTarget(Player[i].BoundingRectangle))
                {
                    hasReachedExit = true;
                    break;
                }
            }

            if (levelTime == TimeSpan.Zero)
            {
                // Still want to perform physics on the player.
                for(int i=0; i<Player.Length; i++)
                    Player[i].ApplyPhysics(time);

                isGameFinished = true;
            }
            else if (hasReachedExit)
            {
                isGameFinished = true;
            }
            else
            {
                for (int i = 0; i < Player.Length; i++)
                {
                    Player[i].Update(time);
                }
                levelTime -= time.ElapsedGameTime;
            }

            // Clamp the time remaining at zero.
            if (levelTime < TimeSpan.Zero)
                levelTime = TimeSpan.Zero;
            #endregion

            for (int i = 0; i < Player.Length; i++)
            {
                if (Player[i].Position.Y < camera.Position.Y - 300)
                {
                    camera.MoveUp(camera.Position.Y - 300 - Player[i].Position.Y);
                    CameraChanged();
                }
                else if (Player[i].Position.Y > camera.Position.Y + 200)
                {
                    camera.MoveUp(((camera.Position.Y + 200) - Player[i].Position.Y));
                    CameraChanged();
                }
            }
        }

        public void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin();

            backgroundLayer.Draw(spriteBatch);
            foregroundLayer.Draw(spriteBatch);
            elementLayer.Draw(spriteBatch);
            targetPoint.Draw(gameTime, camera.Position, spriteBatch);

            rope.get_ChainVector(
                new Vector2((float)Player[0].Position.X, Player[0].Position.Y - (camera.Position.Y - 384 - 64)), 
                new Vector2((float)Player[1].Position.X, Player[1].Position.Y - (camera.Position.Y - 384 - 64)));
            rope.Render(spriteBatch);
            
            for (int i = 0; i < Player.Length; i++)
            {
                Player[i].Draw(gameTime, spriteBatch, camera.Position);
            }

            DrawHud(spriteBatch);            
            spriteBatch.End();
        }        
        private void DrawHud(SpriteBatch spriteBatch)
        {
            Rectangle titleSafeArea = screenManager.GraphicsDevice.Viewport.TitleSafeArea;
            Vector2 hudLocation = new Vector2(titleSafeArea.X+20, titleSafeArea.Y+20);
            Vector2 center = new Vector2(titleSafeArea.X + titleSafeArea.Width / 2.0f,
                                         titleSafeArea.Y + titleSafeArea.Height / 2.0f);

            // Draw time remaining. Uses modulo division to cause blinking when the
            // player is running out of time.
            string timeString = "Time to escape: " + levelTime.Minutes.ToString("00") + ":" + levelTime.Seconds.ToString("00");
            Color timeColor;
            
            if (levelTime > warningTime ||
                hasReachedExit ||
                (int)levelTime.TotalSeconds % 2 == 0)
            {
                timeColor = Color.Yellow;
            }
            else
            {
                timeColor = Color.Red;
            }
            DrawShadowedString(spriteBatch, hudFont, timeString, hudLocation, timeColor);

            //// Determine the status overlay message to show.
            Texture2D status = null;
            if (isGameFinished)
            {
                if (hasReachedExit)
                {
                    status = winOverlay;
                }
                else
                {
                    status = loseOverlay;
                }
            } else if(!hasGameStarted)
            {
                status = escapeOverlay;
            }

            if(levelTime < helpTime && !hasGameStarted)
            {
                hasGameStarted = true;
                status = null;
            }

            if (status != null)
            {
                // Draw status message.
                Vector2 statusSize = new Vector2(status.Width, status.Height);
                spriteBatch.Draw(status, center - statusSize / 2, Color.White);
            }
        }

        private void DrawShadowedString(SpriteBatch spriteBatch, SpriteFont font, string value, Vector2 position, Color color)
        {
            spriteBatch.DrawString(font, value, position + new Vector2(1.0f, 1.0f), Color.Black);
            spriteBatch.DrawString(font, value, position, color);
        }

        private void HandleKeyboardInput(float elapsed)
        {
            //check for camera movement
            //float dX = ReadKeyboardAxis(Keyboard.GetState(), Keys.F, Keys.H) *
            //           elapsed * MovementRate;
            //float dY = ReadKeyboardAxis(Keyboard.GetState(), Keys.G, Keys.T) *
            //           elapsed * MovementRate;
            //camera.MoveRight(ref dX);
            //camera.MoveUp(dY);            
        }

        private static float ReadKeyboardAxis(KeyboardState keyState, Keys downKey,
                                              Keys upKey)
        {
            float value = 0;

            if (keyState.IsKeyDown(downKey))
                value -= 1.0f;

            if (keyState.IsKeyDown(upKey))
                value += 1.0f;

            return value;
        }

        public void CameraChanged()
        {
            //set rotation
            backgroundLayer.CameraRotation = camera.Rotation;
            foregroundLayer.CameraRotation = camera.Rotation;
            elementLayer.CameraRotation = camera.Rotation;

            //set zoom
            backgroundLayer.CameraZoom = camera.Zoom;
            foregroundLayer.CameraZoom = camera.Zoom;
            elementLayer.CameraZoom = camera.Zoom;

            //set position
            backgroundLayer.CameraPosition = new Vector2(camera.Position.X, (camera.Position.Y/1.1f) +350 );
            foregroundLayer.CameraPosition = camera.Position;
            elementLayer.CameraPosition = camera.Position;

            //changes have been accounted for, reset the changed value so that this
            //function is not called unnecessarily
            camera.ResetChanged();
        }

        public void Dispose()
        {
            // dispose
        }

        /// <summary>
        /// Gets the collision mode of the tile at a particular location.
        /// This method handles tiles outside of the levels boundries by making it
        /// impossible to escape past the left or right edges, but allowing things
        /// to jump beyond the top of the level and fall off the bottom.
        /// </summary>
        public Tile.TileCollision GetCollision(int x, int y, int PlayerIdentOnDemand)
        {
            // check if a palyer is close to my movement
            for (int i = 0; i < Player.Length; i++)
            {
                if (i != PlayerIdentOnDemand)
                {
                    // touch anther player 
                    if (Player[i].BoundingRectangle.Intersects(Player[PlayerIdentOnDemand].BoundingRectangle))
                    {
                        return Tile.TileCollision.TouchPlayer;
                    }
                     //leve the radius
                    if (((int) Math.Abs(Player[i].Position.X - Player[PlayerIdentOnDemand].Position.X) > 400) ||
                        ((int) Math.Abs(Player[i].Position.Y - Player[PlayerIdentOnDemand].Position.Y) > 400))
                    {
                        return Tile.TileCollision.ChainPlayer;
                        //depend on the other  
                    }
                }
            }

            // Prevent escaping past the level ends.
            if (x < 0 || x > elementColumns - 1)
                return Tile.TileCollision.Impassable;
            // Allow jumping past the level top and falling through the bottom.
            if (y < 0 || y > elementRows - 1)
                return Tile.TileCollision.Impassable;

            Tile tile = elementLayer.GetTile(x, y);
            if (tile == null)
                return Tile.TileCollision.Passable;
            else
                return tile.Collision;
        }

        public Rectangle GetBounds(int x, int y)
        {
            return new Rectangle(x*Tile.Width, y*Tile.Height, Tile.Width, Tile.Height - 20);
        }

        
    }
}